// TabDelimited
// 2007.08.23
// J. J. Weimer
//
// Procedure files to read a tab delimited file and store it
//
//
// Access to Package
//
// -> udStFiLrTabDelimited(udFL)
//		STRUCTURE udFLStandardStructure &udFL
//
// see the structure definition and User's Manual

#pragma version=1.00
#pragma rtGlobals=1
#pragma IgorVersion=6.0

#include "udFLStandardStructure"
#include "udFLStandardFunctions"

// **********************************************
// Globals and Localizations

Static Structure tabdelimitedglobals
	string version
EndStructure

Static Function Inittabdelimitedglobals(tbd)
	STRUCT tabdelimitedglobals &tbd

	tbd.version="1.00"
End

Static Function udFLTabDInititalize(udFL)
	STRUCT udFLStandardStructure &udFL
	
	udFLZeroStructure(udFL)
	udFLTabDQuery(udFL)
	return 0
end

Static Function udFLTabDQuery(udFL)
	STRUCT udFLStandardStructure &udFL
		
	udFL.mimetype = "TEXT"
	udFL.extensions = ".txt;.dat;"
	udFL.procModes = "Auto;Manual;"
	udFL.itemsType = 4			// returns waves
	udFL.itemsDim[p] = p*0
	udFL.itemsDim[0]=-1		// waves are nominally unscaled numeric
	udFL.setNames=(2+8)		// filename or name+prefix supported
	return 0	
end

// **********************************************
// File Loader

Function udStFiLrTabDelimited(udFL)
	STRUCT udFLStandardStructure &udFL

	variable refNum, ic, nt
	string pathStr, fileStr
	
	switch(udFL.userCtrl)
		case -1:		// initialize		
			udFLTabDInititalize(udFL)		
			return 0
		case 0:		// query
			udFLTabDQuery(udFL)
			return 0
		case 1:		// auto
			if (strlen(udFL.fileList)==0)
				udFLSetError(udFL,alert0a,-1)
				return -1
			endif
			break
		case 2:		// manual
			Open/C="TEXT"/D/R/M="Select the tab-delimited file" refNum
			if (strlen(S_filename)!=0)
				udFL.filelist = S_filename + ";"
			else
				udFLSetError(udFL,"No file selected",-1)
				return -1
			endif
			break
	endswitch
	if (udFL.reportCtrl == 1)
		print "... assuring file paths"
	endif
	if (udFLAssureFilePaths(udFL)<0)
		return -1
	endif
	if (udFL.reportCtrl == 1)
		print "... setting data folder location"
	endif
	string cdf = GetDataFolder(1)
	udFLSetDataFolder(udFL)
	if (strlen(udFL.pathStr)!=0)
		pathStr = udFL.pathStr
		NewPath/O/Q tmpTabDdir, pathStr
	endif
	if (udFL.reportCtrl == 1)
		print "... loading file(s)"
	endif
	nt = ItemsInList(udFL.fileList)
	for (ic=0;ic<nt;ic+=1)
		fileStr=StringFromList(ic,udFL.fileList)
		if (udFL.reportCtrl < 0)
			if (strlen(udFL.pathStr)==0)
				LoadWave/Q/N=TDwave/J/K=0 fileStr
			else
				LoadWave/Q/N=TDwave/J/K=0/P=tmpTabDdir fileStr
			endif
		else
			if (strlen(udFL.pathStr)==0)
				LoadWave/A/J/K=0 fileStr
			else
				LoadWave/A/J/K=0/P=tmpTabDdir fileStr
			endif
		endif
		
		// user data requests scaled wave from (at least) two unscaled waves
		
		variable xs, xe, ni, jc
		string xwave, ywave, wList = ""
		strswitch(udFL.userData)
			case "s":
				ni = ItemsInList(S_wavenames)
				if (ni<2)
					udFLSetError(udFL,"Cannot scale only one wave",0)
					break
				endif
				if (udFL.reportCtrl == 1)
					print "... scaling wave(s)"
				endif
				xwave = StringFromList(0,S_wavenames)
				wave nwave=$xwave
				xs = nwave[0]
				xe = nwave[numpnts(nwave)-1]
				killwaves/Z nwave
				for (jc=1;jc<ni;jc+=1)
					ywave = StringFromList(jc,S_wavenames)
					wave nwave = $ywave
					SetScale/I x, xs, xe, nwave
					xwave = ywave + num2str(ic)
					rename nwave $xwave
					wList += xwave +";"
				endfor
				S_wavenames = wList
		endswitch
		
		// rename waves
		
		switch(udFL.setNames)
			case 1:
				S_wavenames=udFLRenameInput(S_wavenames,"TDwave")
				break
			case 2:
				if (udFL.reportCtrl == 1)
					print "... renaming wave(s)"
				endif
				udFL.namePrefix = ReplaceString(" ",ParseFilePath(3,fileStr,":",0,0),"")
				S_wavenames=udFLRenameInput(S_waveNames,udFL.namePrefix)
				break
			case 8:
				if (udFL.reportCtrl == 1)
					print "... renaming wave(s)"
				endif
				S_wavenames=udFLRenameInput(S_waveNames,udFL.namePrefix)
				break
		endswitch
		udFL.itemsList += S_wavenames
	endfor
	udFL.itemsCount = ItemsInList(udFL.itemsList)
	SetDataFolder $cdf
	return 0
end
